/*
 * Vinyl widget example:
 * Name: Range Slider
 * Developer: Vinyl
 * Date: 4/29/2021
 */
define(function () {

    var install = function (holderElement, context) {
        var sliderElement = holderElement.firstElementChild.firstElementChild;
        var cell = context.getCell();

        // get the parameter from the row. Will use defaults of 0 and 100.
        var min = parseInt(context.getParameter("Min"), 10);
        var max = parseInt(context.getParameter("Max"), 10);

        // step wasn't a parameter in a previous version of this widget, so set
        // it to 1 if the parameter isn't defined.
        try {
            var step = parseFloat(context.getParameter("Step"));
        } catch (e) {
            var step = 1;
        }

        // will determine if the label shows decimal places or not
        var decimalsInLabel = Number.isInteger(step) ? 0 : 2;

        // install the slider library
        var sliderLibrary = context.loadedDependencies[0];
        var wNumb = context.loadedDependencies[1];

        sliderLibrary.create(sliderElement, {
            start: cell.value === null ? 0 : cell.value,
            connect: false,
            range: {
                'min': min,
                'max': max
            },
            step: step,
            pips: {
                mode: 'positions',
                values: [0, 20, 40, 60, 80, 100],
                density: 3,
                stepped: true
            },
            tooltips: [
                wNumb({decimals: decimalsInLabel})
            ],
        });

        sliderElement.noUiSlider.on('change', function (values, handle) {
            // slider change event sets the Vinyl cell value

            var value = values[handle];
            cell.value = value;
        });

        // bind events that Vinyl will fire
        cell.setChangeCallback('disabled', function (state) {
            if (!state) {
                sliderElement.removeAttribute('disabled');
                return;
            }

            sliderElement.setAttribute('disabled', 'true');
        });

        cell.setChangeCallback('value', function (value) {
            if (_.isEqual(sliderElement.noUiSlider.get(), value)) {
                return;
            }

            // if the value changes
            sliderElement.noUiSlider.set(value);
        });

        cell.setChangeCallback('formattedValue', function (value) {
            // set the element title (hover) when this value changes
            sliderElement.setAttribute('title', value);
        });

        context.setDataChangeCallback(function () {
            var updatedMin = parseInt(context.getParameter("Min"), 10);
            var updatedMax = parseInt(context.getParameter("Max"), 10);

            // This version (14.7.0) of the slider doesn't support changing
            // min & max partway through
            /*sliderElement.noUiSlider.updateOptions({
                range: {
                    'min': updatedMin,
                    'max': updatedMax
                }
            });*/
        });
    };

    var uninstall = function (holderElement, context) {
        var sliderElement = holderElement.childNodes[0];

        sliderElement.noUiSlider.off('change');

        sliderElement.noUiSlider.destroy();
    };

    return {
        callbacks: {
            events: {
                // adding and removing
                install: install,
                uninstall: uninstall
            }
        }
    };
});